@@ -1,6 +1,6 @@
  /***************************************************************************\
 |*                                                                           *|
-|*       Copyright 1993-2010 NVIDIA, Corporation.  All rights reserved.      *|
+|*       Copyright 1993-2011 NVIDIA, Corporation.  All rights reserved.      *|
 |*                                                                           *|
 |*     NOTICE TO USER:   The source code  is copyrighted under  U.S. and     *|
 |*     international laws.  Users and possessors of this source code are     *|
@@ -11,7 +11,7 @@
 |*     tion and  internal comments to the code,  notices to the end user     *|
 |*     as follows:                                                           *|
 |*                                                                           *|
-|*       Copyright 1993-2010 NVIDIA, Corporation.  All rights reserved.      *|
+|*       Copyright 1993-2011 NVIDIA, Corporation.  All rights reserved.      *|
 |*                                                                           *|
 |*     NVIDIA, CORPORATION MAKES NO REPRESENTATION ABOUT THE SUITABILITY     *|
 |*     OF  THIS SOURCE  CODE  FOR ANY PURPOSE.  IT IS  PROVIDED  "AS IS"     *|
@@ -26,7 +26,7 @@
 |*     OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOURCE CODE.     *|
 |*                                                                           *|
 |*     U.S. Government  End  Users.   This source code  is a "commercial     *|
-|*     item,"  as that  term is  defined at  48 C.F.R. 2.101 (OCT 1995),     *|
+|*     item,"  as that  term is  defined at  8 C.F.R. 2.101 (OCT 1995),     *|
 |*     consisting  of "commercial  computer  software"  and  "commercial     *|
 |*     computer  software  documentation,"  as such  terms  are  used in     *|
 |*     48 C.F.R. 12.212 (SEPT 1995)  and is provided to the U.S. Govern-     *|
@@ -84,7 +84,7 @@ typedef double             NvF64; /* IEEE Double Precision (S1E11M52)        */
 #define NvU16_HI08(n) ((NvU8)(((NvU16)(n)) >> 8))
 #define NvU16_LO08(n) ((NvU8)((NvU16)(n)))
 
-// Macro to build a NvU16 from msb and lsb bytes. 
+// Macro to build a NvU16 from msb and lsb bytes.
 #define NvU16_BUILD(msb, lsb)  (((msb) << 8)|(lsb))
 
 #if defined(macosx) || defined(MACOS) || defined(NV_MACINTOSH) || \
@@ -130,6 +130,10 @@ typedef NvU8 NvBool;
 #define NV_TRUE           ((NvBool)(0 == 0))
 #define NV_FALSE          ((NvBool)(0 != 0))
 
+/* Tristate type: NV_TRUE, NV_FALSE, NV_INDETERMINATE */
+typedef NvU8 NvTristate;
+#define NV_INDETERMINATE  ((NvTristate) 2)
+
 /* Macros to extract the low and high parts of a 64-bit unsigned integer */
 /* Also designed to work if someone happens to pass in a 32-bit integer */
 #define NvU64_HI32(n)     ((NvU32)((((NvU64)(n)) >> 32) & 0xffffffff))
@@ -175,12 +179,12 @@ typedef NvU32           NvLength; /* length to agree with sizeof             */
 
 // XXX Obsolete -- get rid of me...
 typedef NvP64 NvP64_VALUE_T;
-typedef NvUPtr NV_UINTPTR_T;
 #define NvP64_LVALUE(n)   (n)
 #define NvP64_SELECTOR(n) (0)
 
 /* Useful macro to hide required double cast */
-#define NV_PTR_TO_NvP64(n) (NvP64)(NV_UINTPTR_T)(n)
+#define NV_PTR_TO_NvP64(n) (NvP64)(NvUPtr)(n)
+#define NV_SIGN_EXT_PTR_TO_NvP64(p) ((NvP64)(NvS64)(NvSPtr)(p))
 #define KERNEL_POINTER_TO_NvP64(p) ((NvP64)(uintptr_t)(p))
 
 /* obsolete stuff  */
@@ -271,6 +275,12 @@ typedef unsigned char *PUCHAR;
 #define NV_DECLARE_ALIGNED(TYPE_VAR, ALIGN) __align(ALIGN) TYPE_VAR
 #endif
 
+// NVRM_IMPORT is defined on windows for nvrm4x build (nvrm4x.lib).
+#if (defined(NV_WINDOWS) && defined(NVRM4X_BUILD))
+#define NVRM_IMPORT __declspec(dllimport)
+#else
+#define NVRM_IMPORT
+#endif
 
  /***************************************************************************\
 |*                       Function Declaration Types                          *|
@@ -309,17 +319,22 @@ typedef unsigned char *PUCHAR;
     #define NV_INLINE
     #endif
 
-    #if defined(__GNUC__)
-        // GCC 3.1 and beyond support the always_inline function attribute.
-        #if (__GNUC__ > 3) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
-        #define NV_FORCEINLINE __attribute__((always_inline)) __inline__
-        #else
-        #define NV_FORCEINLINE __inline__
+    /* Don't force inline on DEBUG builds -- it's annoying for debuggers. */
+    #if !defined(DEBUG)
+        #if defined(__GNUC__)
+            // GCC 3.1 and beyond support the always_inline function attribute.
+            #if (__GNUC__ > 3) || (__GNUC__ == 3 && __GNUC_MINOR__ >= 1)
+            #define NV_FORCEINLINE __attribute__((always_inline)) __inline__
+            #else
+            #define NV_FORCEINLINE __inline__
+            #endif
+        #elif defined(__arm) && (__ARMCC_VERSION >= 220000)
+            // RVDS 2.2 also supports forceinline, but ADS 1.2 does not
+            #define NV_FORCEINLINE __forceinline
+        #else /* defined(__GNUC__) */
+            #define NV_FORCEINLINE NV_INLINE
         #endif
-    #elif defined(__arm) && (__ARMCC_VERSION >= 220000)
-        // RVDS 2.2 also supports forceinline, but ADS 1.2 does not
-        #define NV_FORCEINLINE __forceinline
-    #else /* defined(__GNUC__) */
+    #else
         #define NV_FORCEINLINE NV_INLINE
     #endif
 
@@ -358,23 +373,31 @@ typedef unsigned char *PUCHAR;
 typedef NvS16                                                       NvSFXP11_5;
 typedef NvS16                                                       NvSFXP4_12;
 typedef NvS16                                                        NvSFXP8_8;
+typedef NvS32                                                       NvSFXP8_24;
 typedef NvS32                                                      NvSFXP16_16;
+typedef NvS32                                                      NvSFXP20_12;
 typedef NvS32                                                       NvSFXP24_8;
+typedef NvS32                                                       NvSFXP27_5;
+typedef NvS32                                                       NvSFXP29_3;
 
 typedef NvU16                                                       NvUFXP4_12;
+typedef NvU32                                                       NvUFXP8_24;
+typedef NvU32                                                       NvUFXP9_23;
 typedef NvU32                                                      NvUFXP16_16;
+typedef NvU32                                                      NvUFXP20_12;
+typedef NvU32                                                       NvUFXP24_8;
 
 /*!
  * Utility macros used in converting between signed integers and fixed-point
  * notation.
- * 
+ *
  * - COMMON - These are used by both signed and unsigned.
  */
 #define NV_TYPES_FXP_INTEGER(x, y)                              ((x)+(y)-1):(y)
 #define NV_TYPES_FXP_FRACTIONAL(x, y)                                 ((y)-1):0
 #define NV_TYPES_FXP_FRACTIONAL_MSB(x, y)                       ((y)-1):((y)-1)
-#define NV_TYPES_FXP_FRACTIONAL_MSB_ONE(x, y)                        0x00000001
-#define NV_TYPES_FXP_FRACTIONAL_MSB_ZERO(x, y)                       0x00000000
+#define NV_TYPES_FXP_FRACTIONAL_MSB_ONE                              0x00000001
+#define NV_TYPES_FXP_FRACTIONAL_MSB_ZERO                             0x00000000
 /*!
  * - UNSIGNED - These are only used for unsigned.
  */
@@ -396,6 +419,11 @@ typedef NvU32                                                      NvUFXP16_16;
  * Conversion macros used for converting between integer and fixed point
  * representations.  Both signed and unsigned variants.
  *
+ * Warning:
+ * Note that most of the macros below can overflow if applied on values that can
+ * not fit the destination type.  It's caller responsibility to ensure that such
+ * situations will not occur.
+ *
  * Some conversions perform some commonly preformed tasks other than just
  * bit-shifting:
  *
@@ -404,7 +432,7 @@ typedef NvU32                                                      NvUFXP16_16;
  *   non-integer values.
  *
  * - _ROUNDED:
- *   For fixed-point -> integer we add rounding to integer values. 
+ *   For fixed-point -> integer we add rounding to integer values.
  *
  * Unsigned:
  */
@@ -423,8 +451,9 @@ typedef NvU32                                                      NvUFXP16_16;
                     ((NvUFXP##x##_##y) (fxp)))))
 
 #define NV_TYPES_UFXP_X_Y_TO_U32_ROUNDED(x, y, fxp)                           \
-    ((NV_TYPES_UFXP_X_Y_TO_U32(x, y, (fxp))) +                                \
-     (DRF_VAL(_TYPES, _FXP, _FRACTIONAL_MSB((x), (y)), (fxp))))
+    (NV_TYPES_UFXP_X_Y_TO_U32(x, y, (fxp)) +                                  \
+        !!DRF_VAL(_TYPES, _FXP, _FRACTIONAL_MSB((x), (y)),                    \
+            ((NvUFXP##x##_##y) (fxp))))
 
 /*!
  * Signed:
@@ -450,10 +479,70 @@ typedef NvU32                                                      NvUFXP16_16;
                     NV_TYPES_SFXP_S32_SIGN_EXTENSION_POSITIVE((x), (y))))))
 
 #define NV_TYPES_SFXP_X_Y_TO_S32_ROUNDED(x, y, fxp)                           \
-    ((NV_TYPES_SFXP_X_Y_TO_S32(x, y, (fxp))) +                                \
-     (DRF_VAL(_TYPES, _FXP, _FRACTIONAL_MSB((x), (y)), (fxp))))
+    (NV_TYPES_SFXP_X_Y_TO_S32(x, y, (fxp)) +                                  \
+        !!DRF_VAL(_TYPES, _FXP, _FRACTIONAL_MSB((x), (y)),                    \
+            ((NvSFXP##x##_##y) (fxp))))
 
 
+/*!
+ * Macros representing the single-precision IEEE 754 floating point format for
+ * "binary32", also known as "single" and "float".
+ *
+ * http://en.wikipedia.org/wiki/Single_precision_floating-point_format
+ *
+ * _SIGN
+ *     Single bit representing the sign of the number.
+ * _EXPONENT
+ *     Unsigned 8-bit number representing the exponent value by which to scale
+ *     the mantissa.
+ *     _BIAS - The value by which to offset the exponent to account for sign.
+ * _MANTISSA
+ *     Explicit 23-bit significand of the value.  When exponent != 0, this is an
+ *     implicitly 24-bit number with a leading 1 prepended.  This 24-bit number
+ *     can be conceptualized as FXP 9.23.
+ *
+ * With these definitions, the value of a floating point number can be
+ * calculated as:
+ *     (-1)^(_SIGN) *
+ *         2^(_EXPONENT - _EXPONENT_BIAS) *
+ *         (1 + _MANTISSA / (1 << 23))
+ */
+#define NV_TYPES_SINGLE_SIGN                                               31:31
+#define NV_TYPES_SINGLE_SIGN_POSITIVE                                 0x00000000
+#define NV_TYPES_SINGLE_SIGN_NEGATIVE                                 0x00000001
+#define NV_TYPES_SINGLE_EXPONENT                                           30:23
+#define NV_TYPES_SINGLE_EXPONENT_ZERO                                 0x00000000
+#define NV_TYPES_SINGLE_EXPONENT_BIAS                                 0x0000007F
+#define NV_TYPES_SINGLE_MANTISSA                                            22:0
+
+
+/*!
+ * Helper macro to return a IEEE 754 single-precision value's mantissa as an
+ * unsigned FXP 9.23 value.
+ *
+ * @param[in] single   IEEE 754 single-precision value to manipulate.
+ *
+ * @return IEEE 754 single-precision values mantissa represented as an unsigned
+ *     FXP 9.23 value.
+ */
+#define NV_TYPES_SINGLE_MANTISSA_TO_UFXP9_23(single)                           \
+    ((NvUFXP9_23)(FLD_TEST_DRF(_TYPES, _SINGLE, _EXPONENT, _ZERO, single) ?    \
+                    NV_TYPES_U32_TO_UFXP_X_Y(9, 23, 0) :                       \
+                    (NV_TYPES_U32_TO_UFXP_X_Y(9, 23, 1) +                      \
+                        DRF_VAL(_TYPES, _SINGLE, _MANTISSA, single))))
+
+/*!
+ * Helper macro to return an IEEE 754 single-precision value's exponent,
+ * including the bias.
+ *
+ * @param[in] single   IEEE 754 single-precision value to manipulate.
+ *
+ * @return Signed exponent value for IEEE 754 single-precision.
+ */
+#define NV_TYPES_SINGLE_EXPONENT_BIASED(single)                                \
+    ((NvS32)(DRF_VAL(_TYPES, _SINGLE, _EXPONENT, single) -                     \
+        NV_TYPES_SINGLE_EXPONENT_BIAS))
+
 #include "xapi-sdk.h"       /* XAPIGEN sdk macros for C */
 
 
